@@ -43,10 +43,14 @@
 
 #define TRACKER_DAEMON_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TRACKER_TYPE_DAEMON, TrackerDaemonPrivate))
 
+#define TRACKER_TYPE_G_STRV_ARRAY  (dbus_g_type_get_collection ("GPtrArray", G_TYPE_STRV))
+
 typedef struct {
 	TrackerConfig	 *config;
 	TrackerProcessor *processor;
 	DBusGProxy	 *indexer_proxy;
+
+	GTimeVal          last_stats_time;
 	GHashTable       *last_stats;
 } TrackerDaemonPrivate;
 
@@ -58,7 +62,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static void tracker_daemon_finalize (GObject	 *object);
+static void tracker_daemon_finalize (GObject *object);
 
 static guint signals[LAST_SIGNAL] = {0};
 
@@ -120,10 +124,10 @@ tracker_daemon_class_init (TrackerDaemonClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      tracker_marshal_VOID__BOXED,
+			      g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE,
 			      1,
-			      G_TYPE_STRV);
+			      TRACKER_TYPE_G_STRV_ARRAY);
 
 	g_type_class_add_private (object_class, sizeof (TrackerDaemonPrivate));
 }
@@ -305,31 +309,113 @@ tracker_daemon_get_stats (TrackerDaemon		 *object,
 			  DBusGMethodInvocation  *context,
 			  GError		**error)
 {
-	TrackerDBInterface *iface;
-	TrackerDBResultSet *result_set;
-	guint		    request_id;
-	GPtrArray	   *values = NULL;
+	TrackerDaemonPrivate *priv;
+	TrackerDBInterface   *iface;
+	TrackerDBResultSet   *result_set;
+	guint		      request_id;
+	GPtrArray            *values;
+	GTimeVal	      now;
+	gboolean              use_cache;
 
 	request_id = tracker_dbus_get_next_request_id ();
 
 	tracker_dbus_request_new (request_id,
 				  "DBus request to get daemon service stats");
 
-	/* Here it doesn't matter which one we ask, as long as it has common.db
-	 * attached. The service ones are cached connections, so we can use
-	 * those instead of asking for an individual-file connection (like what
-	 * the original code had)
-	 */
+	priv = TRACKER_DAEMON_GET_PRIVATE (object);
 
-	/* iface = tracker_db_manager_get_db_interfaceX (TRACKER_DB_COMMON); */
+	values = g_ptr_array_new ();
 
-	iface = tracker_db_manager_get_db_interface_by_service (TRACKER_DB_FOR_FILE_SERVICE);
+	/* Use the cache if we have requests stats recently. */
+	g_get_current_time (&now);
 
-	result_set = tracker_data_manager_exec_proc (iface, "GetStats", 0);
-	values = tracker_dbus_query_result_to_ptr_array (result_set);
+	if (priv->last_stats_time.tv_sec == 0) {
+		use_cache = FALSE;
+	} else {
+		use_cache = now.tv_sec - priv->last_stats_time.tv_sec < 60;
+	}
 
-	if (result_set) {
-		g_object_unref (result_set);
+	if (use_cache) {
+		GHashTableIter iter;
+		gpointer key, value;
+
+		g_message ("Using cache for stats");
+		
+		g_hash_table_iter_init (&iter, priv->last_stats);
+		while (g_hash_table_iter_next (&iter, &key, &value)) {
+			GStrv         strv;
+			const gchar  *service_type;
+			gint          count;
+			
+			service_type = key;
+			count = GPOINTER_TO_INT (value);
+
+			strv = g_new (gchar*, 3);
+			strv[0] = g_strdup (service_type);
+			strv[1] = g_strdup_printf ("%d", count);
+			strv[2] = NULL;
+
+			g_ptr_array_add (values, strv);
+		}
+	} else {
+		GPtrArray *stats, *parent_stats;
+		gint i;
+
+		g_message ("Using database for stats (cache is %ld seconds old)",
+			   now.tv_sec - priv->last_stats_time.tv_sec);
+
+		/* Here it doesn't matter which one we ask, as long as
+		 * it has common.db attached. The service ones are
+		 * cached connections, so we can use those instead of
+		 * asking for an individual-file connection (like what
+		 * the original code had).
+		 */
+		iface = tracker_db_manager_get_db_interface_by_service (TRACKER_DB_FOR_FILE_SERVICE);
+		
+		result_set = tracker_data_manager_exec_proc (iface, "GetStats", 0);
+		stats = tracker_dbus_query_result_to_ptr_array (result_set);
+		
+		if (result_set) {
+			g_object_unref (result_set);
+		}
+		
+		result_set = tracker_data_manager_exec_proc (iface, "GetStatsForParents", 0);
+		parent_stats = tracker_dbus_query_result_to_ptr_array (result_set);
+		
+		if (result_set) {
+			g_object_unref (result_set);
+		}
+
+		/* Concatenate stats */
+		for (i = 0; i < stats->len; i++) {
+			g_ptr_array_add (values, g_ptr_array_index (stats, i));
+		}
+		
+		for (i = 0; i < parent_stats->len; i++) {
+			g_ptr_array_add (values, g_ptr_array_index (parent_stats, i));
+		}
+
+		/* Update local cache */
+		for (i = 0; i < values->len; i++) {
+			gchar       **p;
+			const gchar  *service_type = NULL;
+			gint          new_count;
+
+			p = g_ptr_array_index (values, i);
+			service_type = p[0];
+			new_count = atoi (p[1]);
+
+			g_hash_table_replace (priv->last_stats, 
+					      g_strdup (service_type), 
+					      GINT_TO_POINTER (new_count));
+		}
+
+		g_ptr_array_free (parent_stats, TRUE);
+		g_ptr_array_free (stats, TRUE);
+
+		priv->last_stats_time = now;
+
+		g_message ("Updated stats cache time to now");
 	}
 
 	dbus_g_method_return (context, values);
@@ -565,7 +651,10 @@ tracker_daemon_signal_statistics (void)
 	TrackerDaemonPrivate *priv;
 	TrackerDBInterface   *iface;
 	TrackerDBResultSet   *result_set;
-	GPtrArray	     *new_stats;
+	GPtrArray	     *stats, *parent_stats;
+	GPtrArray            *values;
+	GTimeVal              now;
+	gint                  i;
 
 	daemon = tracker_dbus_get_object (TRACKER_TYPE_DAEMON);
 	priv = TRACKER_DAEMON_GET_PRIVATE (daemon);
@@ -576,13 +665,36 @@ tracker_daemon_signal_statistics (void)
 	 * lookup the in a to compare in b, just compare index based.
 	 * Maybe we want to change this nonetheless later?
 	 */
+	g_message ("Using database for stats for accurate signal");
+
 	result_set = tracker_data_manager_exec_proc (iface, "GetStats", 0);
-	new_stats = tracker_dbus_query_result_to_ptr_array (result_set);
+	stats = tracker_dbus_query_result_to_ptr_array (result_set);
 
 	if (result_set) {
 		g_object_unref (result_set);
 	}
 
+	result_set = tracker_data_manager_exec_proc (iface, "GetStatsForParents", 0);
+	parent_stats = tracker_dbus_query_result_to_ptr_array (result_set);
+
+	if (result_set) {
+		g_object_unref (result_set);
+	}
+	
+	/* Concatenate stats */
+	values = g_ptr_array_new ();
+	
+	for (i = 0; i < stats->len; i++) {
+		g_ptr_array_add (values, g_ptr_array_index (stats, i));
+	}
+
+	for (i = 0; i < parent_stats->len; i++) {
+		g_ptr_array_add (values, g_ptr_array_index (parent_stats, i));
+	}
+
+	g_ptr_array_free (parent_stats, TRUE);
+	g_ptr_array_free (stats, TRUE);
+
 	/* There are 3 situations here:
 	 *  - 1. No new stats
 	 *       Action: Do nothing
@@ -595,64 +707,54 @@ tracker_daemon_signal_statistics (void)
 	g_message ("Checking for statistics changes and signalling clients...");
 
 	/* Situation #1 */
-	if (!new_stats) {
+	if (!values || values->len < 1) {
 		g_message ("  No new statistics, doing nothing");
 		return;
 	}
 
 	if (g_hash_table_size (priv->last_stats) < 1) {
-		GStrv strv;
 		gint i;
 
 		/* Situation #2 */
-		strv = g_new0 (gchar*, new_stats->len + 1);
-		strv[new_stats->len] = NULL;
-
 		g_message ("  No previous statistics");
 
-		for (i = 0; i < new_stats->len; i++) {
+		for (i = 0; i < values->len; i++) {
 			const gchar **p;
 			const gchar  *service_type = NULL;
 			gint          new_count;
 
-			p = g_ptr_array_index (new_stats, i);
+			p = g_ptr_array_index (values, i);
 
-			service_type = p[1];
-			new_count = atoi (p[0]);
+			service_type = p[0];
+			new_count = atoi (p[1]);
 			
 			if (!service_type) {
 				continue;
 			}
 
-			g_hash_table_insert (priv->last_stats, 
-					     g_strdup (service_type), 
-					     GINT_TO_POINTER (new_count));
-
-			/* GStrv for signal emission */
 			g_message ("  Adding '%s' with count:%d", 
 				   service_type,
 				   new_count);
-			strv[i] = g_strdup (service_type);
+			g_hash_table_insert (priv->last_stats, 
+					     g_strdup (service_type), 
+					     GINT_TO_POINTER (new_count));
 		}
 
 		/* Emit signal */
-		g_signal_emit (daemon, signals[SERVICE_STATISTICS_UPDATED], 0, strv);
-		g_strfreev (strv);
+		g_signal_emit (daemon, signals[SERVICE_STATISTICS_UPDATED], 0, values);
 	} else {
-		GStrv strv = NULL;
-		GSList *l = NULL;
 		gint i;
 
 		/* Situation #3 */
-		for (i = 0; i < new_stats->len; i++) {
-			const gchar **p;
+		for (i = 0; i < values->len; i++) {
+			gchar       **p;
 			const gchar  *service_type = NULL;
 			gpointer      data;
 			gint          old_count, new_count;
 
-			p = g_ptr_array_index (new_stats, i);
-			service_type = p[1];
-			new_count = atoi (p[0]);
+			p = g_ptr_array_index (values, i);
+			service_type = p[0];
+			new_count = atoi (p[1]);
 
 			if (!service_type) {
 				continue;
@@ -668,27 +770,35 @@ tracker_daemon_signal_statistics (void)
 					   old_count,
 					   new_count - old_count);
 
-				l = g_slist_prepend (l, (gpointer) service_type);
-
 				g_hash_table_replace (priv->last_stats, 
 						      g_strdup (service_type), 
 						      GINT_TO_POINTER (new_count));
+			} else {
+				/* Remove from values since the value is the same */
+				g_strfreev (p);
+				g_ptr_array_remove (values, p);
+
+				/* Decrement i since we are about to
+				 * increment it and we just removed
+				 * an item. Otherwise we miss items.
+				 */
+				i--;
 			}
 		}
 
-		if (l) {
-			l = g_slist_reverse (l);
-			strv = tracker_dbus_slist_to_strv (l);
-			g_slist_free (l);
-
-			g_signal_emit (daemon, signals[SERVICE_STATISTICS_UPDATED], 0, strv);
-			g_strfreev (strv);
+		if (values->len > 0) {
+			g_signal_emit (daemon, signals[SERVICE_STATISTICS_UPDATED], 0, values);
 		} else {
 			g_message ("  No changes in the statistics");
-
 		}
 	}
 
-	g_ptr_array_foreach (new_stats, (GFunc) g_strfreev, NULL);
-	g_ptr_array_free (new_stats, TRUE);
+	/* Update time for last stats */
+	g_get_current_time (&now);
+	priv->last_stats_time = now;
+
+	g_message ("Updated stats cache time to now");
+
+	g_ptr_array_foreach (values, (GFunc) g_strfreev, NULL);
+	g_ptr_array_free (values, TRUE);
 }
